@@ -23,6 +23,8 @@ _VERBOSE_VERY = False  # overrides the other and prints all logs to stdout
 
 from imgstore.stores.utils.mixins import MIXINS
 
+logger = logging.getLogger(__name__)
+
 # note: frame_idx always refers to the frame_idx within the chunk
 # whereas frame_index refers to the global frame_index from (0, frame_count]
 
@@ -122,7 +124,7 @@ class _ImgStore(AbstractImgStore, ReadingStore, WritingStore, *MIXINS):
 
     # noinspection PyShadowingBuiltins
     def __init__(self, basedir, mode, fps=25, imgshape=None, imgdtype=np.uint8, chunksize=None, metadata=None,
-                 encoding=None, write_encode_encoding=None, format=None, index=None):
+                 encoding=None, write_encode_encoding=None, format=None, index=None, **kwargs):
         if mode not in self._supported_modes:
             raise ValueError('mode not supported')
 
@@ -143,9 +145,10 @@ class _ImgStore(AbstractImgStore, ReadingStore, WritingStore, *MIXINS):
         self._encode_image = None
         self._uuid = None
 
-        self._metadata = {}
+        self._metadata = {"framerate": fps, **kwargs}
         self._user_metadata = {}
         self._frame_metadata = {}
+        self._write_imgshape = ()
 
         self._tN = self._t0 = time.time()
 
@@ -166,6 +169,7 @@ class _ImgStore(AbstractImgStore, ReadingStore, WritingStore, *MIXINS):
         self._chunk_n = 0
         self._chunk_current_frame_idx = -1
         self._chunk_n_and_chunk_paths = None
+        self._last_img = None
 
         # file pointer and filename of a file which can be used to store additional data per frame
         # (this is only created if data is actually stored)
@@ -213,14 +217,29 @@ class _ImgStore(AbstractImgStore, ReadingStore, WritingStore, *MIXINS):
             assert self._chunk_n == 0
             self.frame_number = np.nan  # we haven't read any frames yet
 
+
+    def _calculate_image_shape(self, imgshape, fmt):
+        _imgshape = list(imgshape)
+        # bitwise and with -2 truncates downwards to even
+        _imgshape[0] = int(_imgshape[0]) & -2
+        _imgshape[1] = int(_imgshape[1]) & -2
+        return tuple(_imgshape)
+
     def get_chunk(self, chunk):
         """
         Place the store so the next frame is the first frame of the chunk
         """
+        self._log.debug(f"{self}.get_chunk({chunk})")
+        assert chunk is not None
         first_fn=self._index.get_chunk_metadata(chunk)["frame_number"][0]
         img, (frame_number, frame_time) = self.get_image(max(0, first_fn))
         return img, (frame_number, frame_time)
     
+    @property
+    def frame_metadata(self):
+        if len(self._frame_metadata) == 0:
+            self._frame_metadata=self._index.get_all_metadata()
+        return self._frame_metadata
 
     @property
     def index_db_path(self):
@@ -230,12 +249,6 @@ class _ImgStore(AbstractImgStore, ReadingStore, WritingStore, *MIXINS):
     def index_db_exists(self):
         return os.path.exists(self.index_db_path)
 
-
-    # noinspection PyShadowingBuiltins,PyMethodMayBeStatic
-    def _calculate_written_image_shape(self, imgshape, fmt):
-        # TODO: This can surely be removed?
-        return imgshape
-
     def frame_number2frame_index(self, frame_number, chunk=None):
         if chunk:
             try:
@@ -306,4 +319,4 @@ class _ImgStore(AbstractImgStore, ReadingStore, WritingStore, *MIXINS):
 
         self._frame_n = 0
 
-__all__ = [_ImgStore]
\ No newline at end of file
+__all__ = [_ImgStore]
